use crate::ast::{FullyQualifiedName, Identifier};
use std::borrow::Cow;

pub trait Named {
    fn name(&self) -> Cow<'_, str>;
}

impl Named for Identifier {
    fn name(&self) -> Cow<'_, str> {
        Cow::Borrowed(&self.name)
    }
}

impl Named for FullyQualifiedName {
    fn name(&self) -> Cow<'_, str> {
        if self.identifiers.len() == 1 {
            self.identifiers[0].name()
        } else {
            let mut acc = String::new();
            for (i, identifier) in self.identifiers.iter().enumerate() {
                acc += &identifier.name();
                if i < self.identifiers.len() - 1 {
                    acc += "::";
                }
            }
            Cow::Owned(acc)
        }
    }
}
