use deimos::ast::build::build_ast;
use deimos::ast::pretty_print::PrettyPrint;
use deimos::parser::{DeimosParser, Rule};
use deimos::util::indent_writer::IndentWriter;
use pest::Parser;
use std::path::PathBuf;

pub fn pretty_print_parse(path: &PathBuf) {
    let src = std::fs::read_to_string(path).expect(&format!("Unable to read {:?}", path));
    let parse_result = DeimosParser::parse(Rule::CompilationUnit, &src);
    match parse_result {
        Ok(mut pairs) => {
            let compilation_unit_pair = pairs.next().unwrap();
            let compilation_unit = build_ast(compilation_unit_pair);
            let mut indent_writer = IndentWriter::new(0, "  ", Box::new(std::io::stdout()));
            compilation_unit
                .pretty_print(&mut indent_writer)
                .expect("Unable to pretty-print.");
        }
        Err(e) => {
            eprintln!("{}", e);
        }
    }
}
